import requests
import pyotp
from pprint import pprint


def parse_response(response: requests.Response):
    print(f"Status Code: {response.status_code}")
    print("Response JSON:")
    if response.status_code == 200:
        pprint(response.json())
    else:
        print(response.text)
        raise Exception(f"Error: {response.status_code}")


class pgsdk:
    def __init__(
        self,
        base_url: str = "http://localhost:22437",
        api_key: str = "",
        verbose: bool = True,
    ):
        self.base_url = base_url
        self.headers = {}
        if api_key:
            api_key = api_key.replace("Bearer ", "").replace("bearer ", "")
            self.headers = {"Authorization": api_key}
        self.verbose = verbose

    def login(self, email, otp):
        response = requests.post(
            f"{self.base_url}/v1/login",
            json={"email": email, "token": otp},
        )
        if self.verbose:
            parse_response(response)
        response = response.json()
        if "detail" in response:
            detail = response["detail"]
            if "?token=" in detail:
                token = detail.split("token=")[1]
                self.headers = {"Authorization": token}
                print(f"Log in at {detail}")
                return token

    def register_user(self, email, first_name, last_name):
        login_response = requests.post(
            f"{self.base_url}/v1/user",
            json={
                "email": email,
                "first_name": first_name,
                "last_name": last_name,
            },
        )
        if self.verbose:
            parse_response(login_response)
        response = login_response.json()
        if "otp_uri" in response:
            mfa_token = str(response["otp_uri"]).split("secret=")[1].split("&")[0]
            totp = pyotp.TOTP(mfa_token)
            return self.login(email=email, otp=totp.now())
        else:
            return response

    def user_exists(self, email):
        response = requests.get(f"{self.base_url}/v1/user/exists?email={email}")
        if self.verbose:
            parse_response(response)
        return response.json()

    def update_user(self, **kwargs):
        response = requests.put(
            f"{self.base_url}/v1/user",
            headers=self.headers,
            json={**kwargs},
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_user(self):
        response = requests.get(f"{self.base_url}/v1/user", headers=self.headers)
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_cases(self):
        response = requests.get(f"{self.base_url}/v1/cases", headers=self.headers)
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_case(self, case_id):
        response = requests.get(
            f"{self.base_url}/v1/cases/{case_id}", headers=self.headers
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_case_queries(self, case_id):
        response = requests.get(
            f"{self.base_url}/v1/cases/{case_id}/queries", headers=self.headers
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def new_case_query(self, case_id, query_string):
        response = requests.post(
            f"{self.base_url}/v1/cases/{case_id}/queries",
            headers=self.headers,
            json={"query_string": query_string},
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_case_query(self, query_id):
        response = requests.get(
            f"{self.base_url}/v1/queries/{query_id}", headers=self.headers
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def create_case(self, case_description, case_name, case_number):
        response = requests.post(
            f"{self.base_url}/v1/cases",
            headers=self.headers,
            json={
                "caseDescription": case_description,
                "caseName": case_name,
                "caseNumber": case_number,
            },
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def process_case(self, case_id):
        response = requests.post(
            f"{self.base_url}/v1/cases/{case_id}/process",
            headers=self.headers,
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def delete_case(self, case_id):
        response = requests.post(
            f"{self.base_url}/v1/cases/{case_id}/delete",
            headers=self.headers,
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def update_case(self, case_id, **kwargs):
        response = requests.post(
            f"{self.base_url}/v1/cases/{case_id}",
            headers=self.headers,
            json=kwargs,
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def upload_document(self, case_id, file_path):
        with open(file_path, "rb") as file:
            response = requests.post(
                f"{self.base_url}/v1/cases/{case_id}/documents",
                headers=self.headers,
                files={"file": file},
            )
        if self.verbose:
            parse_response(response)
        return response.json()

    def upload_document_from_buffer(self, case_id, file_name, file_buffer):
        response = requests.post(
            f"{self.base_url}/v1/cases/{case_id}/documents",
            headers=self.headers,
            files={"file": (file_name, file_buffer)},
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def get_case_documents(self, case_id):
        response = requests.get(
            f"{self.base_url}/v1/cases/{case_id}/documents",
            headers=self.headers,
        )
        if self.verbose:
            parse_response(response)
        return response.json()

    def delete_document(self, document_id):
        response = requests.post(
            f"{self.base_url}/v1/documents/{document_id}/delete",
            headers=self.headers,
        )
        if self.verbose:
            parse_response(response)
        return response.json()
